local bridge = {}
bridge.__index = bridge
local function new()
    return setmetatable({decks={},saved={},water=1,modeText='Simulate',showForces=false,showGrid=false,previousMove={},joints={},beams={},active=false,gridX=50,gridY=50,x1=0,x2=0,y=0,xOff=0,yOff=0,updateFrequency=300,updateCount=0}, bridge)   --,
end

function bridge:draw()
    love.graphics.setColor(0,0,0,255)
    for i,v in ipairs(self.beams) do
        v:draw()
    end
    love.graphics.setPoint(7,'smooth')
    love.graphics.setColor(170,50,50,255)
    for i,v in ipairs(self.joints) do
        v:draw()
    end
    love.graphics.setPoint(4,'rough')
    if self.showGrid then
        self:drawGrid(self.x1,self.x2,self.y)
    end
end

function bridge:update(dt)
        if (self.active) then
                world.obj:update(dt)
                for i,v in ipairs(self.beams) do
                    v:update(dt)
                end
            self.updateCount=self.updateCount+1
            if (self.updateCount>=self.updateFrequency) then
                for i,v in ipairs(self.joints) do
                    v:reduceLoad(10)  --Remove residual load left over from base
                end
            end
        end
end
function bridge:drawGrid(x1,x2,y)
    local yTop=y-10*self.gridY
    local yBottom=y+10*self.gridY
    love.graphics.setColor(100,100,100,120)
    love.graphics.setLineWidth(1)
    for i=0,math.floor((x2-x1)/self.gridX) do
        local xTemp=x1+i*self.gridX
        love.graphics.line(xTemp,yTop,xTemp,yBottom)
        for j=-10,10 do
            local yTemp=y+j*self.gridY
            love.graphics.line(x1,yTemp,x2,yTemp)
        end
    end
end
function bridge:toggleGrid()
    self.showGrid=not self.showGrid
    if self.showGrid then
        self:setWaterValues()
    end
end
function bridge:setWaterValues()
    self.x1=currentMap.wStart
    self.x2=currentMap.wEnd
    self.xOff=self.x1%self.gridX
    self.y=currentMap.soilLevel[math.floor(self.x1/4)]*4
    self.yOff=self.y%self.gridY
end


function bridge:reset(fullReset,noRefund)
   for i=1,#self.beams do
      self.beams[1]:destroy()
   end
   local ii=1
   for i=1,#self.joints do
       if self.joints[ii].mobile=='dynamic' or fullReset then
           self.joints[ii]:destroy()
       else
           ii=ii+1
       end
   end
   if not noRefund then
   mainBase:resetMetalUsedForBridge()
    end
   self.modeText='Simulate'
   self.active=false
   gi:updateBridgeMode()
end

function bridge:findClosestJoint(x,y,distance)
    if self.showGrid then
        x,y= math.round((x)/self.gridX)*self.gridX+self.xOff,math.round((y)/self.gridY)*self.gridY+self.yOff
    end
    for i,v in ipairs(self.joints) do
        local tempX=v.body:getX()
        local tempY=v.body:getY()
        if(mag(tempX-x,tempY-y)<distance) then
            return tempX,tempY,v
        end
    end

    return x,y,false
end
function bridge:findClosestBeam(x,y,distance)
for i,v in ipairs(self.beams) do
    local x1,y1,x2,y2=v.joint:getAnchors()
    if mag((x1+x2)/2-x,(y2+y1)/2-y)<=distance then
        return x,y,v
    end
end
return x,y,false

end
function bridge:checkDeck()
    local decks={}
    local waterStart=currentMap.wStart
    local waterEnd=currentMap.wEnd
    for i,v in ipairs(self.beams) do
        if v.building.name=='Deck' then
           local x1,y1,x2,y2=v.joint:getAnchors()
           if x1>x2 then
               gi:logText('!!! Broke!','bridge')
           end
           --     table.insert(decks,{x2,y2,x1,y1,v})
           --else
           table.insert(decks,{x1,y1,x2,y2,v})
           --end
        end
    end
    if #decks<2 then
        return false
    end

    table.sort(decks,function(a,b)
       return a[1]<b[1]
    end)

    local currentX=decks[1][3]
    local currentY=decks[1][4]
    local span=decks[1][3]-decks[1][1]
    local newDecks={decks[1]}
    for i,v in ipairs(decks) do
       if v[1]==currentX and currentY==v[2] then
            currentX=v[3]
            currentY=v[4]
            table.insert(newDecks,v)
       end
    end
    if currentX>waterEnd and decks[1][1]<waterStart then
        self.decks=newDecks
        return true
    else
        return false
    end
end
function bridge:undo()
    buildingToPlace=-1
    gi.cursor='normal'
    if type(self.beams[#self.beams])~='nil' then
        self:deleteWithJoint(self.beams[#self.beams])
    end
end

function bridge:deleteWithJoint(beam)
    beam:destroy()
    if beam.j1.mobile=='dynamic' then
        if self:checkNumberConnections(beam.j1)==0 then
            beam.j1:destroy()
        end
    end
    if beam.j2.mobile=='dynamic' then
        if self:checkNumberConnections(beam.j2)==0 then
            beam.j2:destroy()
        end
    end
end

function bridge:checkNumberConnections(joint)
    local connections=0
    for i,v in ipairs(self.beams) do
        if v.j1==joint or v.j2==joint then
            connections=connections+1
        end
    end
    return connections
end
function bridge:simulate()
    self.decks={}
    local check=self:checkDeck()
    if not check then
       gi:logText('You need a single continous deck which crosses the water.','bridge')
        return
    end
    self:saveBridge()
    self.modeText='Cancel Simulation'
    self.active=true
    gi:updateBridgeMode()
    self:setWaterValues()
    mainBase:startSim(self.decks)


end
function bridge:simSuccess()
    self.modeText='Success!'
    gi:updateBridgeMode()
    self:reset(true,true)
    gi:openDialog({
        {'sub','LevelComplete!'},
        {'textDesc','Area secure.'},
        {'sub','Reward'},
        {'text','CPU',currentMap.reward},
        {'functionButton','Back to Base',function() loadBaseLevel(true) end},
    })
end
function bridge:toNextLevel()
    self:reset(true,true)
end
function bridge:simFail()
    self.modeText='Sim Failed'
    self.active=false
    gi:updateBridgeMode()
end
function bridge:cancelSimulate()

    self:reset(true)
    self:loadBridge(self.saved)
    self.modeText='Simulate'
    self.active=false
    gi:updateBridgeMode()
    mainBase:endSim()
end
function bridge:saveBridge()
   local save={}
   save.joints={}
   for i,v in ipairs(self.joints) do
       --if v.mobile=='dynamic' then
            save.joints[i]={v.body:getX(),v.body:getY(),v.body:getMass()}
       --end
   end
   save.beams={}
   for i,v in ipairs(self.beams) do
       save.beams[i]={v.j1.body:getX(),v.j1.body:getY(),v.j2.body:getX(),v.j2.body:getY(),v.metal,v.building}
   end
   self.saved=save
end

function bridge:loadBridge(b)
    for i,v in ipairs(self.saved.joints) do
        self.joints[i]=joint(v[1],v[2],v[3])
    end
    for i,v in ipairs(self.saved.beams) do
        local j1=self:findSavedJoint(v[1],v[2])
        local j2=self:findSavedJoint(v[3],v[4])
        self.beams[i]=beam({j1,j2},v[5],v[6],1)
    end
end
function bridge:findSavedJoint(x,y)
    for i,v in ipairs(self.saved.joints) do
        if v[1]==x and v[2]==y then
            return self.joints[i]
        end
    end

end
function bridge:getConnectedToJoints(j1,j2)
    local found=0
    for i,v in ipairs(self.beams) do
        if (v.j1==j1 or v.j1==j2) and (v.j2==j1 or v.j2==j2) then
           return v
        end
    end
    return false
end

return setmetatable({new = new},
    {__call = function(_, ...) return new(...) end})